library(ggplot2)
library(scales)
load <- function(path = "") {
  
  if (path == "") {
    path = paste(sep = "/", getwd(),"CSVs")
  }
  
  readTable <- function(filePath) {
    return(read.table(filePath, sep = "|", header = T, comment.char = "\\", stringsAsFactors = F))
  }
  
  getFilesList <- function(pattern, recursive) {
    return (list.files(path = path, pattern = pattern, full.names = TRUE, recursive = recursive))
  }
  
  # load CSV files from given path
  allCSVs = getFilesList("*.csv", TRUE)
  i = 0
  for (csvFilePath in allCSVs) {
    if (!grepl(".testdifferences|.compileerrors", csvFilePath)) {
      cat(sprintf("Reading file %s\n", csvFilePath))
      tbl = readTable(csvFilePath)
      tableName = gsub("-", "_", substr(basename(csvFilePath), 1, nchar(basename(csvFilePath)) - 4))
      if (nrow(tbl) > 0) {
        tbl$ProjectName <- gsub(".report|.trees|.precondviolations|.blockgapsinfo|.exprgapsinfo", "", tableName)
      }
      assign(tableName, tbl, envir = .GlobalEnv)
    }
  }
  
  unionTables = function(postfix) {
    allObjectNames = ls(envir = .GlobalEnv)
    allObjectsWithPostfix = lapply(allObjectNames[grepl(paste(".", postfix, "$", sep = ""), allObjectNames)], get)
    return(do.call(rbind, allObjectsWithPostfix))
  }
  
  allReports <<- unionTables("report")
  allTrees <<- unionTables("trees")
  allPreconditionViolations <<- unionTables("precondviolations")
  allBlockGapsInfo <<- unionTables("blockgapsinfo")
  allExpressionGapsInfo <<- unionTables("exprgapsinfo")
  
  reportsTreesJoined <<- merge(allReports, allTrees, by=c("GroupID", "PairID", "ProjectName"))
  reportsTreesJoined$TotalTime <<- reportsTreesJoined$SubtreeMatchingWallNanoTime + reportsTreesJoined$PDGMappingWallNanoTime
  reportsTreesJoined$Average <<- (reportsTreesJoined$X.StatementsInCloneFragment1 + reportsTreesJoined$X.StatementsInCloneFragment2) / 2

  KEYS <<- c("GroupID", "PairID", "TreeID", "ProjectName")
  
  PRECONDITION_VIOLATION_NAMES <<-  c("EXPRESSION_DIFFERENCE_CANNOT_BE_PARAMETERIZED",
                             "EXPRESSION_DIFFERENCE_IS_FIELD_UPDATE",
                             "EXPRESSION_DIFFERENCE_IS_VOID_METHOD_CALL",
                             "EXPRESSION_DIFFERENCE_IS_METHOD_CALL_THROWING_EXCEPTION_WITHIN_MATCHED_TRY_BLOCK",
                             "INFEASIBLE_UNIFICATION_DUE_TO_VARIABLE_TYPE_MISMATCH",
                             "INFEASIBLE_UNIFICATION_DUE_TO_MISSING_MEMBERS_IN_THE_COMMON_SUPERCLASS",
                             "INFEASIBLE_UNIFICATION_DUE_TO_PASSED_ARGUMENT_TYPE_MISMATCH",
                             "UNMATCHED_STATEMENT_CANNOT_BE_MOVED_BEFORE_OR_AFTER_THE_EXTRACTED_CODE",
                             "UNMATCHED_STATEMENT_CANNOT_BE_MOVED_BEFORE_THE_EXTRACTED_CODE_DUE_TO_CONTROL_DEPENDENCE",
                             "UNMATCHED_BREAK_STATEMENT",
                             "UNMATCHED_CONTINUE_STATEMENT",
                             "UNMATCHED_RETURN_STATEMENT",
                             "UNMATCHED_THROW_STATEMENT",
                             "UNMATCHED_EXCEPTION_THROWING_STATEMENT_NESTED_WITHIN_MATCHED_TRY_BLOCK",
                             "MULTIPLE_RETURNED_VARIABLES",
                             "UNEQUAL_NUMBER_OF_RETURNED_VARIABLES",
                             "SINGLE_RETURNED_VARIABLE_WITH_DIFFERENT_TYPES",
                             "BREAK_STATEMENT_WITHOUT_LOOP",
                             "CONTINUE_STATEMENT_WITHOUT_LOOP",
                             "CONDITIONAL_RETURN_STATEMENT",
                             "SWITCH_CASE_STATEMENT_WITHOUT_SWITCH",
                             "SUPER_CONSTRUCTOR_INVOCATION_STATEMENT",
                             "SUPER_METHOD_INVOCATION_STATEMENT",
                             "MULTIPLE_UNMATCHED_STATEMENTS_UPDATE_THE_SAME_VARIABLE",
                             "INFEASIBLE_REFACTORING_DUE_TO_UNCOMMON_SUPERCLASS",
                             "INFEASIBLE_REFACTORING_DUE_TO_ZERO_MATCHED_STATEMENTS",
                             "NOT_ALL_POSSIBLE_EXECUTION_FLOWS_END_IN_RETURN")
  
  REFACTORING_TYPES <<- c(
    "EXTRACT_LOCAL_METHOD",
    "PULL_UP_TO_EXISTING_SUPERCLASS",
    "PULL_UP_TO_NEW_INTERMEDIATE_SUPERCLASS_EXTENDING_COMMON_INTERNAL_SUPERCLASS",
    "PULL_UP_TO_NEW_INTERMEDIATE_SUPERCLASS_IMPLEMENTING_COMMON_INTERNAL_INTERFACE",
    "PULL_UP_TO_NEW_SUPERCLASS_EXTENDING_COMMON_EXTERNAL_SUPERCLASS",
    "PULL_UP_TO_NEW_SUPERCLASS_IMPLEMENTING_COMMON_EXTERNAL_INTERFACE",
    "PULL_UP_TO_NEW_SUPERCLASS_EXTENDING_OBJECT",
    "EXTRACT_STATIC_METHOD_TO_NEW_UTILITY_CLASS",
    "INFEASIBLE")
}

getWithoutCompileErrorsTable <- function() {
  return(reportsTreesJoined[reportsTreesJoined$HadCompileErrorsAfterRefactoring == "false" & 
                       reportsTreesJoined$RefactoringWasOK == "true" &
                       reportsTreesJoined$X.PreconditionViolations != 0, c(KEYS, "Average")])
}

load()
rm(load)